define("epi/shell/widget/DeferredDropDownButton", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/on",
    // Parent class
    "dijit/form/DropDownButton"
], function(
    declare,
    lang,
    Deferred,
    on,
    // Parent class
    DropDownButton
) {

    return declare([DropDownButton], {

        _loaded: false,

        isLoaded: function() {
            // summary:
            //      Indicates whether the dropdown content has loaded.
            // tag:
            //      public
            return this._loaded && !!this.dropDown;
        },

        loadDropDown: function() {
            // summary:
            //      Loads the drop down asynchronously and resolves when the load is complete.
            // returns: Promise
            //      Promise for the drop down widget that resolves when
            //      the drop down is loaded.
            // tags:
            //      protected
            var self = this,
                dropDown = this.dropDown,
                deferred = new Deferred();

            on.once(dropDown, "loaded", function() {
                self._loaded = true;
                deferred.resolve();
            });

            dropDown.startup();

            return deferred.promise;
        },

        loadAndOpenDropDown: function() {
            // summary:
            //      Creates the drop down if it doesn't exist and then opens the drop down.
            // returns: Promise
            //      Promise for the drop down widget that resolves when
            //      the drop down is created and loaded.
            // tags:
            //      protected
            var self = this,
                dropDown = this.dropDown;

            if (this.isLoaded()) {
                return new Deferred().resolve(this.openDropDown());
            }

            return this.loadDropDown().then(function() {
                self.openDropDown();
                if (dropDown.focus) {
                    self.defer(lang.hitch(dropDown, "focus"), 1);
                }
            });
        }
    });
});
